{$I-}
PROGRAM FixStat;

CONST VERSION = '0.2';

VAR Filename : STRING;
    Header   : RECORD
          magic    : ARRAY[1..8] OF CHAR;
          nentries : INTEGER;
          recsize  : INTEGER;
    END;
    Buffer   : RECORD
          turn     : INTEGER;
          date     : ARRAY[1..10] OF CHAR;
          time     : ARRAY[1..8] OF CHAR;
          rest     : ARRAY[1..1024] OF CHAR;
    END;
    VPAbuf   : RECORD
          turnsig  : LONGINT;
          datasize : LONGINT;
          turn     : INTEGER;
          date     : ARRAY[1..10] OF CHAR;
          time     : ARRAY[1..8] OF CHAR;
          rest     : ARRAY[1..88] OF CHAR;
    END;
    file_type : (fPCC, fVPA);
    F : FILE;

CONST VPA_TURNSIG = $4E525554; { 'TURN' }

PROCEDURE Help;
BEGIN
  Writeln('FIXSTAT v' + Version + ' - PCC Score / VPA Database Manipulation - 2002 by Stefan Reuther'#13#10);

  Writeln('Usage: FIXSTAT dir\stat.cc [turn number]'#13#10+
          '       FIXSTAT dir\vpaX.db [turn number]'#13#10);

  Writeln('With turn number, will remove all records from that turn and'#13#10+
          'more recent turns from the file.'#13#10);

  Writeln('Without turn number, will list all turn numbers along with'#13#10+
          'their timestamps.'#13#10);

  Writeln('Report bugs to <Streu@gmx.de>');
  Halt;
END;

PROCEDURE Error(CONST S:STRING);
BEGIN
  Writeln('FIXSTAT: ', S);
  Halt(1);
END;

PROCEDURE Open;
VAR R:WORD;
    vpahead:ARRAY[1..15] OF CHAR;
BEGIN
  Assign(F, Filename);
  Reset(F, 1);
  IF IOResult<>0 THEN Error(Filename + ': can''t open');
  Blockread(F, Header, Sizeof(Header), R);
  IF (R=Sizeof(Header)) THEN BEGIN
    IF (Header.magic='CC-Stat'#26) AND (Header.nentries>0) AND (Header.recsize>=108) THEN BEGIN
      file_type := fPCC;
      IF Header.recsize > Sizeof(Buffer) THEN Error(Filename + ': file format not supported (record size too big)');
      IF Filesize(F) < LONGINT(Header.recsize) * Header.nentries + Sizeof(Header) THEN Error(Filename + ': file too short');
      Exit;
    END ELSE IF Header.magic='VPA Data' THEN BEGIN
      Seek(F, 0);
      Blockread(F, vpahead, Sizeof(vpahead), R);
      IF (R=Sizeof(vpahead)) AND (vpahead='VPA Database'#13#10#6) THEN BEGIN
        file_type := fVPA;
        Exit;
      END;
    END;
  END;
  Error(Filename + ': invalid file format');
END;

PROCEDURE List;
VAR i:INTEGER;
    expect:INTEGER;
    r:WORD;
BEGIN
  FileMode := 0;
  Open;
  Writeln('Turn     Date       Time');
  Writeln('----  ----------  --------');
  expect:=1;
  CASE file_type OF
    fPCC: FOR i:=1 TO Header.nentries DO BEGIN
      Blockread(F, Buffer, Header.recsize);
      Write(Buffer.turn : 4, '  ', Buffer.date, '  ', Buffer.time);
      IF expect<>Buffer.turn THEN Writeln('  (I would have expected turn ', expect, ')')
                             ELSE Writeln;
      expect := Buffer.turn+1;
    END;
    fVPA: REPEAT
      Blockread(F, VPAbuf, Sizeof(VPAbuf), R);
      IF R=0 THEN Break;
      IF (R<>Sizeof(VPAbuf)) OR (VPAbuf.turnsig<>VPA_TURNSIG) THEN Error(filename + ': file is damaged');
      Write(VPABuf.turn : 4, '  ', VPAbuf.date, '  ', VPAbuf.time);
      IF expect<>VPAbuf.turn THEN Writeln('  (I would have expected turn ', expect, ')')
                             ELSE Writeln;
      expect := VPAbuf.turn+1;
      Seek(F, FilePos(F) + VPAbuf.datasize);
    UNTIL FALSE;
  END;
  Close(F);
END;

PROCEDURE Remove;
VAR i       : INTEGER;
    nr, k   : INTEGER;
    L       : LONGINT;
    R       : WORD;
    cuthere : BOOLEAN;
BEGIN
  Val(ParamStr(2), nr, k);
  IF (k<>0) OR (nr<=0) THEN Error('Invalid turn number specified');
  Open;
  CASE file_type OF
    fPCC: FOR i:=1 TO Header.nentries DO BEGIN
      L := FilePos(F);
      Blockread(F, Buffer, Header.recsize);
      IF Buffer.turn >= nr THEN BEGIN
        IF i=1 THEN BEGIN
          Writeln('That truncation would make the file empty -- deleting it');
          Close(F);
          Erase(F);
          Exit;
        END;
        Header.nentries := i-1;
        Seek(F, 0);
        Blockwrite(F, Header, Sizeof(Header));
        Seek(F, L);
        Truncate(F);
        Writeln('Truncated file after ', i-1, ' records.');
        Break;
      END;
    END;
    fVPA: BEGIN
      i := 1;
      REPEAT
        L := FilePos(F);
        Blockread(F, VPAbuf, Sizeof(VPAbuf), R);
        IF R=0 THEN Break;
        cuthere := FALSE;
        IF R<>Sizeof(VPAbuf) THEN BEGIN
          cuthere := TRUE;
          Writeln(filename, ': file is damaged, truncating bogus part');
        END ELSE cuthere := VPAbuf.turn >= nr;
        IF cuthere THEN BEGIN
          IF i=1 THEN BEGIN
            Writeln('That truncation would make the file empty -- deleting it');
            Close(F);
            Erase(F);
            Exit;
          END;
          Seek(F, L);
          Truncate(F);
          Writeln('Truncated file after ', i-1, ' records.');
          Break;
        END;
        Inc(i);
        Seek(F, FilePos(F) + VPAbuf.datasize);
      UNTIL FALSE;
    END;
  END;
  Close(F);
END;

BEGIN
  Filename := ParamStr(1);
  CASE ParamCount OF
    1: IF (Filename='/?') OR (Filename='-?') THEN Help ELSE List;
    2: Remove;
  ELSE Help;
  END;
END.
